@@ -1,5 +1,5 @@
 # typedload
-# Copyright (C) 2018 Salvo "LtWorf" Tomaselli
+# Copyright (C) 2018-2019 Salvo "LtWorf" Tomaselli
 #
 # typedload is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,11 +22,22 @@ from enum import Enum
 from typing import Dict, List, NamedTuple, Optional, Set, Tuple, Union
 import unittest
 
-from typedload import dataloader, load, dump
+from typedload import dataloader, load, dump, typechecks
 
 
 class TestDataclassLoad(unittest.TestCase):
 
+    def test_is_dataclass(self):
+        @dataclass
+        class A:
+            pass
+        class B(NamedTuple):
+            pass
+        assert typechecks.is_dataclass(A)
+        assert not typechecks.is_dataclass(List[int])
+        assert not typechecks.is_dataclass(Tuple[int, int])
+        assert not typechecks.is_dataclass(B)
+
     def test_factory_load(self):
         @dataclass
         class A:
