@@ -194,6 +194,34 @@ class TestEnum(unittest.TestCase):
         assert loader.load(['2', 1], Tuple[TestEnum, TestEnum]) == (TestEnum.LABEL2, TestEnum.LABEL1)
 
 
+class TestForwardRef(unittest.TestCase):
+
+    def test_known_refs(self):
+        class Node(NamedTuple):
+            value: int = 1
+            next: Optional['Node'] = None
+        l = {'next': {}, 'value': 12}
+        loader = dataloader.Loader()
+        assert loader.load(l, Node) == Node(value=12,next=Node())
+
+    def test_disable(self):
+        class A(NamedTuple):
+            i: 'int'
+        loader = dataloader.Loader(frefs=None)
+        with self.assertRaises(Exception):
+            loader.load(3, A)
+
+    def test_add_fref(self):
+        class A(NamedTuple):
+            i: 'alfio'
+        loader = dataloader.Loader()
+        with self.assertRaises(ValueError):
+            loader.load({'i': 3}, A)
+        loader.frefs['alfio'] = int
+        assert loader.load({'i': 3}, A) == A(3)
+
+
+
 class TestLoaderIndex(unittest.TestCase):
 
     def test_removal(self):
@@ -203,3 +231,54 @@ class TestLoaderIndex(unittest.TestCase):
         loader.handlers.pop(loader.index(int))
         with self.assertRaises(TypeError):
             loader.load(3, int)
+
+
+class TestExceptions(unittest.TestCase):
+
+    def test_index(self):
+        loader = dataloader.Loader()
+        try:
+            loader.load([1, 2, 3, 'q'], List[int])
+        except Exception as e:
+            assert e.trace[-1].annotation[1] == 3
+
+        try:
+            loader.load(['q', 2], Tuple[int,int])
+        except Exception as e:
+            assert e.trace[-1].annotation[1] == 0
+
+        try:
+            loader.load({'q': 1}, Dict[int,int])
+        except Exception as e:
+            assert e.trace[-1].annotation[1] == 'q'
+
+    def test_attrname(self):
+        class A(NamedTuple):
+            a: int
+        class B(NamedTuple):
+            a: A
+            b: int
+        loader = dataloader.Loader()
+
+        try:
+            loader.load({'a': 'q'}, A)
+        except Exception as e:
+            assert e.trace[-1].annotation[1] == 'a'
+
+        try:
+            loader.load({'a':'q','b': {'a': 1}}, B)
+        except Exception as e:
+            assert e.trace[-1].annotation[1] == 'a'
+
+        try:
+            loader.load({'a':3,'b': {'a': 'q'}}, B)
+        except Exception as e:
+            assert e.trace[-1].annotation[1] == 'a'
+
+    def test_typevalue(self):
+        loader = dataloader.Loader()
+        try:
+            loader.load([1, 2, 3, 'q'], List[int])
+        except Exception as e:
+            assert e.value == 'q'
+            assert e.type_ == int
